/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 


#include "fft.h"
#include "audio_thread.h"
#include "audio_common.h"

#include <pthread.h>
#include <unistd.h>
#include <glib.h>
#include <linux/soundcard.h>
#include <math.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stropts.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <string.h>
#include <sys/un.h>

#include <stdio.h>
#include <errno.h>

int terminate = 0;
pthread_mutex_t terminate_mutex = PTHREAD_MUTEX_INITIALIZER ;
pthread_t tid = 0;

/*
 * Fourier transform from samples to frequency
 *
 */
void calc_freq(gint16 *dest, gint16 *src) {
	static fft_state *state ;
	gfloat tmp_out[WIDTH+1];
	gint i;
	
	if(!state) {
		state = fft_init();
	}

	fft_perform(src,tmp_out,state);
	
	for(i = 0; i < WIDTH; i++) {
		dest[i] = ((gint)sqrt(tmp_out[i + 1])) >> 8;
		if (dest[i] < 0) dest[i] = 0 ;
	}

}



/*
 *
 *
 *
 */
void *_audio_thread(void *user_data) {	
	int arg;      /* argument for ioctl calls */
	int status;   /* return status of system calls */
	int dsp_fd ;
	
	int sd, ret ;
	struct sockaddr_un servaddr ;
		
	
	// buffer to read into
	gint16 buf[CHANNELS*WIDTH*2] ;
	
	// de interlaced sound samples
	gint16 data[CHANNELS][WIDTH*2] ;
	
	// frequency data
	gint16 freq[CHANNELS][WIDTH] ;
	
	gint16 *ptr ;
	int i;
	
	// open sound device 
	dsp_fd = open("/dev/dsp", O_RDONLY);
	if (dsp_fd < 0) {
    	printf("open of /dev/dsp failed");
    	return NULL;
 	}

	// set sampling parameters 
	arg = SIZE;      // sample size 
	status = ioctl(dsp_fd, SOUND_PCM_WRITE_BITS, &arg);
	if (status == -1)
    printf("SOUND_PCM_WRITE_BITS ioctl failed");
	if (arg != SIZE)
 		printf("unable to set sample size");

	arg = CHANNELS;  // mono or stereo 
	status = ioctl(dsp_fd, SOUND_PCM_WRITE_CHANNELS, &arg);
	if (status == -1)
		printf("SOUND_PCM_WRITE_CHANNELS ioctl failed");
 	if (arg != CHANNELS)
    	printf("unable to set number of channels");

	arg = RATE;      // sampling rate 
	status = ioctl(dsp_fd, SOUND_PCM_WRITE_RATE, &arg);
	if (status == -1)
		printf("SOUND_PCM_WRITE_RATE ioctl failed");
  
	// setup socket
	sd = socket(AF_LOCAL, SOCK_STREAM, 0) ;
	
	if(! sd) 
		return NULL ;
	
	memset(&servaddr, 0x00, sizeof(servaddr) );
	
	servaddr.sun_family = AF_LOCAL ;
	strcpy(servaddr.sun_path, LOCAL_PATH ) ;
	
	ret = connect(sd, (struct sockaddr *) &servaddr, sizeof(servaddr) ) ;
	
	if(ret) {
		printf("connect failed %s\n", strerror(errno) ) ;
		return NULL ;
	}
	
	while (1) { /* loop forever */	
		
		pthread_mutex_lock(&terminate_mutex) ;
		if(terminate ==1) {
			printf("terminate is 1\n") ;
			terminate = 0;
			pthread_mutex_unlock(&terminate_mutex) ;
			break ;
		}
		
		pthread_mutex_unlock(&terminate_mutex) ;
		
		/* record some sound */
		status = read(dsp_fd, buf, sizeof(buf)); 
		
		if (status != sizeof(buf)) {
			fprintf(stderr, "read wrong number of bytes or read error");
			break ;
		}

		ptr = buf ;
		
		// de interlace left and right samples
		for (i = 0; i < 512; i++) {
			data[0][i] = *ptr;
			ptr++;
			data[1][i] = *ptr;
			ptr++;
		}
		
		// perform fft
		calc_freq(freq[0], data[0]);
		calc_freq(freq[1], data[1]);	

		// send out pipe
		write(sd, freq, sizeof(freq) );
		
	}
	printf("broke out of loop\n") ;
	
	close(dsp_fd) ;
	
	// close socket

	return NULL;

}







/*
 *
 *
 *
 */
int audio_thread_run() {
	int ret = -1;
	
	if(tid ==0 ) {
		ret = pthread_create (&tid, NULL, _audio_thread, NULL);
	}
	return ret;
}

void audio_thread_terminate() {

	pthread_mutex_lock(&terminate_mutex) ;
	printf("setting terminate to 1\n") ;
	terminate = 1;
	pthread_mutex_unlock(&terminate_mutex) ;
	
	pthread_join(tid, NULL) ;
	
	tid = 0;
	
	
}
